@@ -3,12 +3,12 @@
 		the parallel sampling routine
 		for the C versions of the Cuba routines
 		by Thomas Hahn
-		last modified 30 Apr 13 th
+		last modified 25 Sep 13 th
 */
 
 #define MINSLICE 10
 #define MINCORES 1
-//#define MINCORES 2
+/*#define MINCORES 2*/
 
 typedef struct {
   number n, m, i;
@@ -27,9 +27,9 @@ workerini cubaini;
 #define TERM_BLUE "\e[34m"
 #define TERM_RESET "\e[0m\n"
 #define MASTER(s, ...) \
-fprintf(stderr, TERM_RED ROUTINE " master: " s TERM_RESET, __VA_ARGS__)
+fprintf(stderr, TERM_RED ROUTINE " master %d(%d): " s TERM_RESET, core, getpid(), ##__VA_ARGS__)
 #define WORKER(s, ...) \
-fprintf(stderr, TERM_BLUE ROUTINE " worker: " s TERM_RESET, __VA_ARGS__)
+fprintf(stderr, TERM_BLUE ROUTINE " worker %d(%d): " s TERM_RESET, core, getpid(), ##__VA_ARGS__)
 #else
 #define MASTER(s, ...)
 #define WORKER(s, ...)
@@ -71,11 +71,13 @@ static void DoSample(This *t, number n, creal *x, real *f
   else {
     Slice slice;
     int core, abort;
+    number nx;
     char s[128];
 
     t->neval += n;
 
-    slice.m = slice.n = (n + ncores - 1)/ncores;
+    nx = n % ncores;
+    slice.m = slice.n = n/ncores + 1;
     if( VERBOSE > 2 ) {
       sprintf(s, "sampling " NUMBER " points each on %d cores",
         slice.n, ncores);
@@ -105,6 +107,7 @@ static void DoSample(This *t, number n, creal *x, real *f
 
     for( core = 0; core < ncores; ++core ) {
       cint fd = t->child[core];
+      slice.n -= (core == nx);
       MASTER("sending samples (sli:%lu[+" VES_ONLY(NUMBER "w:%lu+")
         NUMBER "x:%lu]) to fd %d",
         sizeof slice, VES_ONLY(slice.n, sizeof *w,)
@@ -117,12 +120,10 @@ static void DoSample(This *t, number n, creal *x, real *f
         x += slice.n*t->ndim;
       }
       slice.i += slice.n;
-      n -= slice.n;
-      slice.n = IMin(slice.n, n);
     }
 
     abort = 0;
-    for( core = ncores; --core >= 0; ) {
+    for( core = 0; core < ncores; ++core ) {
       cint fd = t->child[core];
       readsock(fd, &slice, sizeof slice);
       MASTER("reading samples (sli:%lu[+" NUMBER "f:%lu]) from fd %d",
@@ -171,32 +172,32 @@ typedef struct {
 
 static int Explore(This *t, cint iregion)
 {
-  TYPEDEFREGION;
+  csize_t regionsize = RegionSize;
   Region *region;
   int ireg = iregion, core = t->running;
+  Vector(Totals, totals, NCOMP);
 
   if( t->ncores < MINCORES ) return ExploreSerial(t, iregion);
 
   if( t->running >= ((iregion < 0) ? 1 : t->ncores) ) {
-    Totals totals[NCOMP];
     cint fd = t->child[core = ReadyCore(t)];
     ExploreResult res;
     count comp, succ;
 
     --t->running;
     MASTER("reading res + region (res:%lu+reg:%lu) from fd %d",
-      sizeof res, sizeof(Region), fd);
+      sizeof res, regionsize, fd);
     readsock(fd, &res, sizeof res);
     ireg = res.iregion;
     region = RegionPtr(ireg);
     succ = ireg + region->next;
-    readsock(fd, region, sizeof(Region));
+    readsock(fd, region, regionsize);
     if( --res.nregions > 0 ) {
       region->next = t->nregions - ireg;
       EnlargeRegions(t, res.nregions);
       MASTER("reading regions (%dreg:%lu) from fd %d",
-        res.nregions, sizeof(Region), fd);
-      readsock(fd, RegionPtr(t->nregions), res.nregions*sizeof(Region));
+        res.nregions, regionsize, fd);
+      readsock(fd, RegionPtr(t->nregions), res.nregions*regionsize);
       t->nregions += res.nregions;
 
       RegionPtr(t->nregions-1)->next = succ - t->nregions + 1;
@@ -224,11 +225,11 @@ static int Explore(This *t, cint iregion)
     slice.phase = t->phase;
     region = RegionPtr(iregion);
     MASTER("writing region (sli:%lu+sam:%lu+reg:%lu+tot:%lu) to fd %d",
-      sizeof slice, sizeof(Samples), sizeof(Region),
+      sizeof slice, sizeof(Samples), regionsize,
       t->ncomp*sizeof(Totals), fd);
     writesock(fd, &slice, sizeof slice);
     writesock(fd, &t->samples[region->isamples], sizeof(Samples));
-    writesock(fd, region, sizeof(Region));
+    writesock(fd, region, regionsize);
     writesock(fd, t->totals, t->ncomp*sizeof(Totals));
     region->depth = 0;
     ++t->running;
@@ -240,13 +241,13 @@ static int Explore(This *t, cint iregion)
 
 /*********************************************************************/
 
-static void DoChild(This *t, cint fd)
+static void DoChild(This *t, cint core, cint fd)
 {
   Slice slice;
 
 #ifdef DIVONNE
-  TYPEDEFREGION;
-  Totals totals[NCOMP];
+  csize_t regionsize = RegionSize;
+  Vector(Totals, totals, NCOMP);
   ExploreResult res;
 
   t->totals = totals;
@@ -272,9 +273,9 @@ static void DoChild(This *t, cint fd)
         sizeof slice, VES_ONLY(n, sizeof *w,) n, t->ndim*sizeof *x, fd);
 
 #ifdef DIVONNE
-      if( n > t->nframe ) {
+      if( slice.m > t->nframe ) {
         FrameFree(t);
-        t->nframe = n;
+        t->nframe = slice.m;
         SHM_ONLY(t->shmid = slice.shmid; ShmMap(t) else)
         MemAlloc(t->frame, t->nframe*SAMPLESIZE);
       }
@@ -306,10 +307,10 @@ static void DoChild(This *t, cint fd)
       Samples *samples, psamples;
 
       WORKER("reading region (sli:%lu+sam:%lu+reg:%lu+tot:%lu) from fd %d",
-        sizeof slice, sizeof(Samples), sizeof(Region),
+        sizeof slice, sizeof(Samples), regionsize,
         t->ncomp*sizeof(Totals), fd);
       readsock(fd, &psamples, sizeof(Samples));
-      readsock(fd, RegionPtr(0), sizeof(Region));
+      readsock(fd, t->region, regionsize);
       readsock(fd, totals, t->ncomp*sizeof(Totals));
       t->nregions = 1;
       t->neval = t->neval_opt = t->neval_cut = 0;
@@ -328,15 +329,16 @@ static void DoChild(This *t, cint fd)
       res.nregions = t->nregions;
       res.iregion = slice.i;
       WORKER("writing regions (res:%lu+%dreg:%lu+tot:%lu) to fd %d",
-        sizeof res, t->nregions, sizeof(Region),
+        sizeof res, t->nregions, regionsize,
         t->ncomp*sizeof(Totals), fd);
       writesock(fd, &res, sizeof res);
-      writesock(fd, RegionPtr(0), t->nregions*sizeof(Region));
+      writesock(fd, t->region, t->nregions*regionsize);
       writesock(fd, totals, t->ncomp*sizeof(Totals));
     }
 #endif
   }
 
+  WORKER("wrapping up");
   if( cubaini.exitfun ) cubaini.exitfun(cubaini.exitarg);
 
   exit(0);
@@ -389,10 +391,10 @@ static inline void ForkCores(This *t)
       (pid = fork()) != -1 );
     if( pid == 0 ) {
       close(fd[0]);
-      DoChild(t, fd[1]);
+      DoChild(t, core, fd[1]);
     }
-    MASTER("forked core %d pid %d pipe %d(master) -> %d(worker)",
-      core, pid, fd[0], fd[1]);
+    MASTER("forked pid %d pipe %d(master) -> %d(worker)",
+      pid, fd[0], fd[1]);
     close(fd[1]);
     t->child[core] = fd[0];
   }
@@ -406,14 +408,14 @@ static inline void WaitCores(cThis *t)
     int core;
     pid_t pid;
     for( core = 0; core < t->ncores; ++core ) {
-      MASTER("closing core %d fd %d", core, t->child[core]);
+      MASTER("closing fd %d", t->child[core]);
       close(t->child[core]);
     }
     free(t->child);
     for( core = 0; core < t->ncores; ++core ) {
-      MASTER("waiting for core %d", core);
+      MASTER("waiting for child");
       wait(&pid);
-      MASTER("core %d pid %d terminated", core, pid);
+      MASTER("pid %d terminated", pid);
     }
   }
 }
