@@ -5,32 +5,38 @@
 		then do a main integration over all regions
 		this file is part of Divonne
 		checkpointing by B. Chokoufe
-		last modified 2 May 13 th
+		last modified 5 Aug 13 th
 */
 
 
+typedef struct {
+  signature_t signature;
+  number neval, neval_opt, neval_cut;
+  number nmin, nrand;
+  count iregion, nregions;
+  count phase, iter, pass, size;
+  Totals totals[];
+} State;
+
 static int Integrate(This *t, real *integral, real *error, real *prob)
 {
-  TYPEDEFREGION;
+  StateDecl;
+  csize_t statesize = sizeof(State) + NCOMP*sizeof(Totals);
+  Sized(State, state, statesize);
+  csize_t regionsize = RegionSize;
+  Vector(char, out, 64*NDIM + 256*NCOMP);
 
-  count dim, comp, err, iregion;
+  Totals *tot, *Tot = state->totals + t->ncomp;
+  Bounds *b, *B;
+  Result *res;
+  count comp, err, iregion;
   number nwant;
   real nneed;
   ML_ONLY(number neff;)
   int fail;
-  struct {
-    signature_t signature;
-    number neval, neval_opt, neval_cut;
-    number nmin, nrand;
-    count iregion, nregions;
-    count phase, iter, pass, size;
-    Totals totals[NCOMP];
-  } state;
-  StateDecl;
 
   if( VERBOSE > 1 ) {
-    char s[512];
-    sprintf(s, "Divonne input parameters:\n"
+    sprintf(out, "Divonne input parameters:\n"
       "  ndim " COUNT "\n  ncomp " COUNT "\n"
       "  epsrel " REAL "\n  epsabs " REAL "\n"
       "  flags %d\n  seed %d\n"
@@ -47,12 +53,12 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
       t->border.lower, t->maxchisq, t->mindeviation,
       t->ngiven, t->nextra,
       t->statefile);
-    Print(s);
+    Print(out);
   }
 
   if( BadComponent(t) ) return -2;
   if( BadDimension(t, t->key1) ||
-      BadDimension(t, t->key2) ||   
+      BadDimension(t, t->key2) ||
       ((t->key3 & -2) && BadDimension(t, t->key3)) ) return -1;
 
   FORK_ONLY(t->nframe = 0;)
@@ -65,7 +71,7 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
   if( IsSobol(t->key1) | IsSobol(t->key2) | IsSobol(t->key3) )
     IniRandom(t);
   t->epsabs = Max(t->epsabs, NOTZERO);
-  t->totals = state.totals;
+  t->totals = state->totals;
 
   ForkCores(t);
 
@@ -79,20 +85,18 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
 
   if( StateReadTest(t) ) {
     StateReadOpen(t, fd) {
-      size_t regsize;
-      if( read(fd, &state, sizeof state) != sizeof state ||
-        state.signature != StateSignature(t, 3) ) break;
-      t->nregions = state.nregions;
-      regsize = t->nregions*sizeof(Region);
-      if( st.st_size != sizeof state + regsize ) break;
-      t->neval = state.neval;
-      t->neval_opt = state.neval_opt;
-      t->neval_cut = state.neval_cut;
-      t->nrand = state.nrand;
-      t->phase = state.phase;
-      t->size = state.size;
+      if( read(fd, state, statesize) != statesize ||
+          state->signature != StateSignature(t, 3) ) break;
+      t->nregions = state->nregions;
+      if( st.st_size != statesize + t->nregions*regionsize ) break;
+      t->neval = state->neval;
+      t->neval_opt = state->neval_opt;
+      t->neval_cut = state->neval_cut;
+      t->nrand = state->nrand;
+      t->phase = state->phase;
+      t->size = state->size;
       AllocRegions(t);
-      read(fd, t->voidregion, regsize);
+      StateRead(fd, t->region, t->nregions*regionsize);
     } StateReadClose(t, fd);
 
     if( IsSobol(t->key1) | IsSobol(t->key2) | IsSobol(t->key3) )
@@ -111,19 +115,18 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
 
     t->size = CHUNKSIZE;
     AllocRegions(t);
-    for( dim = 0; dim < t->ndim; ++dim ) {
-      Bounds *b = &RegionPtr(0)->bounds[dim];
+    for( B = (b = t->region->bounds) + t->ndim; b < B; ++b ) {
       b->lower = 0;
       b->upper = 1;
     }
     t->nregions = 1;
 
     t->phase = 1;
-    state.iter = 1;
-    state.pass = 0;
-    state.nmin = INT_MAX;
-    state.iregion = 0;
-    Zap(state.totals);
+    state->iter = 1;
+    state->pass = 0;
+    state->nmin = INT_MAX;
+    state->iregion = 0;
+    FClear(state->totals);
   }
 
   /* Step 1: partition the integration region */
@@ -133,37 +136,31 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
 
     if( ini ) Iterate(t, 0, INIDEPTH, 0, NULL);
 
-    for( ; ; ++state.iter ) {
+    for( ; ; ++state->iter ) {
       Totals *maxtot;
       count valid;
 
-      for( comp = 0; comp < t->ncomp; ++comp ) {
-        Totals *tot = &state.totals[comp];
+      for( tot = state->totals; tot < Tot; ++tot ) {
         tot->avg = tot->spreadsq = 0;
         tot->spread = tot->secondspread = -INFTY;
       }
 
-      for( iregion = 0; iregion < t->nregions; ++iregion ) {
-        Region *region = RegionPtr(iregion);
-        for( comp = 0; comp < t->ncomp; ++comp ) {
-          cResult *r = &region->result[comp];
-          Totals *tot = &state.totals[comp];
-          tot->avg += r->avg;
-          tot->spreadsq += Sq(r->spread);
-          if( r->spread > tot->spread ) {
+      for( iregion = 0; iregion < t->nregions; ++iregion )
+        for( res = RegionResult(RegionPtr(iregion)), tot = state->totals;
+             tot < Tot; ++res, ++tot ) {
+          tot->avg += res->avg;
+          tot->spreadsq += Sq(res->spread);
+          if( res->spread > tot->spread ) {
             tot->secondspread = tot->spread;
-            tot->spread = r->spread;
+            tot->spread = res->spread;
             tot->iregion = iregion;
           }
-          else if( r->spread > tot->secondspread )
-            tot->secondspread = r->spread;
+          else if( res->spread > tot->secondspread )
+            tot->secondspread = res->spread;
         }
-      }
 
-      maxtot = state.totals;
       valid = 0;
-      for( comp = 0; comp < t->ncomp; ++comp ) {
-        Totals *tot = &state.totals[comp];
+      for( maxtot = tot = state->totals, comp = 0; tot < Tot; ++tot, ++comp ) {
         integral[comp] = tot->avg;
         valid += tot->avg == tot->avg;
         if( tot->spreadsq > maxtot->spreadsq ) maxtot = tot;
@@ -171,41 +168,37 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
         error[comp] = tot->spread/t->samples[0].neff;
       }
 
-#define StateWrite(t) \
+#define WriteState(t) \
 if( StateWriteTest(t) ) { \
   StateWriteOpen(t, fd) { \
-    state.signature = StateSignature(t, 3); \
-    state.neval = t->neval; \
-    state.neval_opt = t->neval_opt; \
-    state.neval_cut = t->neval_cut; \
-    state.nrand = t->nrand; \
-    state.nregions = t->nregions; \
-    state.phase = t->phase; \
-    state.size = t->size; \
-    write(fd, &state, sizeof state); \
-    write(fd, t->voidregion, t->nregions*sizeof(Region)); \
+    state->signature = StateSignature(t, 3); \
+    state->neval = t->neval; \
+    state->neval_opt = t->neval_opt; \
+    state->neval_cut = t->neval_cut; \
+    state->nrand = t->nrand; \
+    state->nregions = t->nregions; \
+    state->phase = t->phase; \
+    state->size = t->size; \
+    StateWrite(fd, state, statesize); \
+    StateWrite(fd, t->region, t->nregions*regionsize); \
   } StateWriteClose(t, fd); \
 }
 
-      StateWrite(t);
+      WriteState(t);
 
       if( VERBOSE ) {
-        char s[128 + 64*NCOMP], *p = s;
-
-        p += sprintf(p, "\n"
+        char *oe = out + sprintf(out, "\n"
           "Iteration " COUNT " (pass " COUNT "):  " COUNT " regions\n"
           NUMBER7 " integrand evaluations so far,\n"
           NUMBER7 " in optimizing regions,\n"
           NUMBER7 " in finding cuts",
-          state.iter, state.pass, t->nregions,
+          state->iter, state->pass, t->nregions,
           t->neval, t->neval_opt, t->neval_cut);
-
         for( comp = 0; comp < t->ncomp; ++comp )
-          p += sprintf(p, "\n[" COUNT "] "
+          oe += sprintf(oe, "\n[" COUNT "] "
             REAL " +- " REAL,
             comp + 1, integral[comp], error[comp]);
-
-        Print(s);
+        Print(out);
       }
 
       if( valid == 0 ) goto abort;	/* all NaNs */
@@ -215,11 +208,11 @@ if( StateWriteTest(t) ) { \
       nneed = maxtot->spread/MaxErr(maxtot->avg);
       if( nneed < MAXPRIME ) {
         cnumber n = t->neval + t->nregions*(number)ceil(nneed);
-        if( n < state.nmin ) {
-          state.nmin = n;
-          state.pass = 0;
+        if( n < state->nmin ) {
+          state->nmin = n;
+          state->pass = 0;
         }
-        else if( ++state.pass > t->maxpass && n >= t->mineval ) break;
+        else if( ++state->pass > t->maxpass && n >= t->mineval ) break;
       }
 
       Iterate(t, maxtot->iregion, DEPTH, -1, NULL);
@@ -230,8 +223,7 @@ if( StateWriteTest(t) ) { \
 
 /* nneed = t->samples[0].neff + 1; */
   nneed = 2*t->samples[0].neff;
-  for( comp = 0; comp < t->ncomp; ++comp ) {
-    Totals *tot = &state.totals[comp];
+  for( tot = state->totals; tot < Tot; ++tot ) {
     creal maxerr = MaxErr(tot->avg);
     tot->nneed = tot->spread/maxerr;
     nneed = Max(nneed, tot->nneed);
@@ -261,62 +253,59 @@ if( StateWriteTest(t) ) { \
     SamplesAlloc(t, &t->samples[1]);
 
     if( VERBOSE ) {
-      char s[128];
-      sprintf(s, "\nMain integration on " COUNT
+      sprintf(out, "\nMain integration on " COUNT
         " regions with " NUMBER " samples per region.",
         t->nregions, t->samples[1].neff);
-      Print(s);
+      Print(out);
     }
 
     nlimit = t->maxeval - t->nregions*t->samples[1].n;
     df = 0;
 
 #define CopyPhaseResults(f) \
-  for( comp = 0; comp < t->ncomp; ++comp ) { \
-    PhaseResult *p = &state.totals[comp].phase[f]; \
-    cResult *r = &region->result[comp]; \
-    p->avg = r->avg; \
-    p->err = r->err; \
+  for( res = RegionResult(region), tot = state->totals; tot < Tot; ++res, ++tot ) { \
+    PhaseResult *p = &tot->phase[f]; \
+    p->avg = res->avg; \
+    p->err = res->err; \
   }
 
-#define Var2(f, res) Sq((res)->err ? (res)->err : r->spread/t->samples[f].neff)
+#define Var2(f, r) Sq((r)->err ? (r)->err : res->spread/t->samples[f].neff)
 #define Var(f) Var2(f, &tot->phase[f])
 
-    while( state.iregion < t->nregions ) {
+    while( state->iregion < t->nregions ) {
       Region *region;
-      char s[64*NDIM + 256*NCOMP], *p = s;
+      char *oe = out;
       int todo;
 
 refine:
-      region = RegionPtr(state.iregion);
+      region = RegionPtr(state->iregion);
       CopyPhaseResults(0);
       t->phase = 2;
       region->isamples = 1;
-      t->samples[1].sampler(t, state.iregion);
+      t->samples[1].sampler(t, state->iregion);
       CopyPhaseResults(1);
 
       if( can_adjust )
-        for( comp = 0; comp < t->ncomp; ++comp )
-          state.totals[comp].spreadsq -= Sq(region->result[comp].spread);
+        for( res = RegionResult(region), tot = state->totals;
+             tot < Tot; ++res, ++tot )
+          tot->spreadsq -= Sq(res->spread);
 
       nlimit += t->samples[1].n;
       todo = 0;
 
-      for( comp = 0; comp < t->ncomp; ++comp ) {
-        cResult *r = &region->result[comp];
-        Totals *tot = &state.totals[comp];
-
+      for( res = RegionResult(region), tot = state->totals;
+           tot < Tot; ++tot ) {
         if( t->neval < nlimit ) {
           creal avg2 = tot->phase[1].avg;
           creal diffsq = Sq(avg2 - tot->phase[0].avg);
 
-          if( r->err*tot->nneed > r->spread ||
+          if( res->err*tot->nneed > res->spread ||
               diffsq > Max(t->maxchisq*(Var(0) + Var(1)), EPS*Sq(avg2)) ) {
             if( t->key3 && diffsq > tot->mindevsq ) {
               if( t->key3 == 1 ) {
                 if( VERBOSE > 2 ) Print("\nSplit");
                 t->phase = 1;
-                Iterate(t, state.iregion, POSTDEPTH, 1, state.totals);
+                Iterate(t, state->iregion, POSTDEPTH, 1, state->totals);
 
                 if( can_adjust ) {
                   cnumber nnew = (tot->spreadsq/Sq(MARKMASK) > tot->maxerrsq) ?
@@ -334,11 +323,10 @@ refine:
                       can_adjust = false;
 
                     if( VERBOSE > 2 ) {
-                      char s[128];
-                      sprintf(s, "Sampling remaining " COUNT
+                      sprintf(out, "Sampling remaining " COUNT
                         " regions with " NUMBER " points per region.",
                         t->nregions, t->samples[1].neff);
-                      Print(s);
+                      Print(out);
                     }
                   }
                 }
@@ -352,14 +340,14 @@ refine:
       }
 
       if( can_adjust )
-        for( comp = 0; comp < t->ncomp; ++comp )
-          state.totals[comp].maxerrsq -=
-            Sq(region->result[comp].spread/t->samples[1].neff);
+        for( res = RegionResult(region), tot = state->totals;
+             tot < Tot; ++res, ++tot )
+          tot->maxerrsq -= Sq(res->spread/t->samples[1].neff);
 
       switch( todo ) {
       case 1:	/* get spread right */
         region->isamples = 1;
-        ExploreSerial(t, state.iregion);
+        ExploreSerial(t, state->iregion);
         break;
 
       case 3:	/* sample region again with more points */
@@ -370,25 +358,22 @@ refine:
         }
         t->phase = 3;
         region->isamples = 2;
-        t->samples[2].sampler(t, state.iregion);
-        ExploreSerial(t, state.iregion);
+        t->samples[2].sampler(t, state->iregion);
+        ExploreSerial(t, state->iregion);
         ++region->depth;	/* misused for df here */
         ++df;
       }
 
-      if( VERBOSE > 2 )
-        for( dim = 0; dim < t->ndim; ++dim ) {
-          cBounds *b = &region->bounds[dim];
-          p += sprintf(p,
-            (dim == 0) ? "\nRegion (" REALF ") - (" REALF ")" :
-                         "\n       (" REALF ") - (" REALF ")",
-            b->lower, b->upper);
+      if( VERBOSE > 2 ) {
+        cchar *msg = "\nRegion (" REALF ") - (" REALF ")";
+        for( B = (b = region->bounds) + t->ndim; b < B; ++b ) {
+          oe += sprintf(oe, msg, b->lower, b->upper);
+          msg = "\n       (" REALF ") - (" REALF ")";
         }
+      }
 
-      for( comp = 0; comp < t->ncomp; ++comp ) {
-        Result *r = &region->result[comp];
-        Totals *tot = &state.totals[comp];
-
+      for( tot = state->totals, res = RegionResult(region), comp = 0;
+           tot < Tot; ++tot, ++res ) {
         creal x1 = tot->phase[0].avg;
         creal v1 = Var(0);
         creal x2 = tot->phase[1].avg;
@@ -403,8 +388,8 @@ refine:
         real chiden = v1 + v2;
 
         if( todo == 3 ) {
-          creal x3 = r->avg;
-          creal v3 = Var2(2, r);
+          creal x3 = res->avg;
+          creal v3 = Var2(2, res);
           creal r3 = v2 ? v3/v2 :
             Sq(t->samples[2].neff/(real)t->samples[1].neff);
 
@@ -415,56 +400,50 @@ refine:
           chiden = v1*v2 + v3*chiden;
         }
 
-        avg = LAST ? r->avg : (sigsq *= norm = 1/norm, avg*norm);
+        avg = LAST ? res->avg : (sigsq *= norm = 1/norm, avg*norm);
         if( chisq > EPS ) chisq /= Max(chiden, NOTZERO);
 
         if( VERBOSE > 2 ) {
-#define Out2(f, res) (res)->avg, r->spread/t->samples[f].neff, (res)->err
+#define Out2(f, r) (r)->avg, res->spread/t->samples[f].neff, (r)->err
 #define Out(f) Out2(f, &tot->phase[f])
-          p += sprintf(p, "\n[" COUNT "] "
+          oe += sprintf(oe, "\n[" COUNT "] "
             REAL " +- " REAL "(" REAL ")\n    "
-            REAL " +- " REAL "(" REAL ")", comp + 1, Out(0), Out(1));
-          if( todo == 3 ) p += sprintf(p, "\n    "
-            REAL " +- " REAL "(" REAL ")", Out2(2, r));
-          p += sprintf(p, "  \tchisq " REAL, chisq);
+            REAL " +- " REAL "(" REAL ")", ++comp, Out(0), Out(1));
+          if( todo == 3 ) oe += sprintf(oe, "\n    "
+            REAL " +- " REAL "(" REAL ")", Out2(2, res));
+          oe += sprintf(oe, "  \tchisq " REAL, chisq);
         }
 
         tot->integral += avg;
         tot->sigsq += sigsq;
         tot->chisq += chisq;
 
-        r->avg = avg;
-        r->spread = sqrt(sigsq);
-        r->chisq = chisq;
+        res->avg = avg;
+        res->spread = sqrt(sigsq);
+        res->chisq = chisq;
       }
 
-      if( VERBOSE > 2 ) Print(s);
-      ++state.iregion;
+      if( VERBOSE > 2 ) Print(out);
+      ++state->iregion;
 
-      StateWrite(t);
+      WriteState(t);
     }
 
     df += t->nregions;
 
-    for( comp = 0; comp < t->ncomp; ++comp ) {
-      Totals *tot = &state.totals[comp];
+    for( tot = state->totals, comp = 0; tot < Tot; ++tot, ++comp ) {
       integral[comp] = tot->integral;
       error[comp] = sqrt(tot->sigsq);
       prob[comp] = ChiSquare(tot->chisq, df);
     }
 
     if( VERBOSE > 2 ) {
-      char s[16 + 128*NCOMP], *p = s;
-
-      p += sprintf(p, "\nTotals:");
-
-      for( comp = 0; comp < t->ncomp; ++comp )
-        p += sprintf(p, "\n[" COUNT "] "
+      char *oe = out + sprintf(out, "\nTotals:");
+      for( tot = state->totals, comp = 0; tot < Tot; ++tot, ++comp )
+        oe += sprintf(oe, "\n[" COUNT "] "
           REAL " +- " REAL "  \tchisq " REAL " (" COUNT " df)",
-          comp + 1, integral[comp], error[comp],
-          state.totals[comp].chisq, df);
-
-      Print(s);
+          comp + 1, integral[comp], error[comp], tot->chisq, df);
+      Print(out);
     }
 
     ML_ONLY(neff = 1;)
@@ -472,28 +451,29 @@ refine:
 
 #ifdef MLVERSION
   if( REGIONS ) {
+    Vector(real, bounds, t->ndim*2);
+
     MLPutFunction(stdlink, "List", 2);
+
     MLPutFunction(stdlink, "List", t->nregions);
     for( iregion = 0; iregion < t->nregions; ++iregion ) {
       Region *region = RegionPtr(iregion);
-      cBounds *b = region->bounds;
-      real lower[NDIM], upper[NDIM];
+      cResult *Res;
+      real *d = bounds;
 
-      for( dim = 0; dim < t->ndim; ++dim ) {
-        lower[dim] = b[dim].lower;
-        upper[dim] = b[dim].upper;
+      for( B = (b = region->bounds) + t->ndim; b < B; ++b ) {
+        *d++ = b->lower;
+        *d++ = b->upper;
       }
 
       MLPutFunction(stdlink, "Cuba`Divonne`region", 4);
 
-      MLPutRealList(stdlink, lower, t->ndim);
-      MLPutRealList(stdlink, upper, t->ndim);
+      MLPutRealList(stdlink, bounds, 2*t->ndim);
 
       MLPutFunction(stdlink, "List", t->ncomp);
-      for( comp = 0; comp < t->ncomp; ++comp ) {
-        cResult *r = &region->result[comp];
-        real res[] = {r->avg, r->spread/neff, r->chisq};
-        MLPutRealList(stdlink, res, Elements(res));
+      for( Res = (res = RegionResult(region)) + t->ncomp; res < Res; ++res ) {
+        real r[] = {res->avg, res->spread/neff, res->chisq};
+        MLPutRealList(stdlink, r, Elements(r));
       }
 
       MLPutInteger(stdlink, region->depth + 1);  /* misused for df */
@@ -509,7 +489,7 @@ abort:
   SamplesFree(&t->samples[2]);
   SamplesFree(&t->samples[1]);
   SamplesFree(&t->samples[0]);
-  free(t->voidregion);
+  free(t->region);
   free(t->xgiven);
 
   StateRemove(t);
