@@ -2,7 +2,7 @@
 	decl.h
 		Type declarations
 		this file is part of Divonne
-		last modified 30 Apr 13 th
+		last modified 26 Jul 13 th
 */
 
 
@@ -34,8 +34,18 @@ typedef struct {
   int iregion;
 } Totals;
 
+enum { nrules = 5 };
+      
 typedef struct {
-  void *first, *last;
+  count n;
+  real weight[nrules], scale[nrules], norm[nrules];
+  real gen[];
+} Set;
+
+#define SetSize (sizeof(Set) + t->ndim*sizeof(real))
+
+typedef struct {
+  Set *first, *last;
   real errcoeff[3];
   count n;
 } Rule;
@@ -58,6 +68,30 @@ typedef struct {
 
 typedef const Errors cErrors;
 
+typedef struct {
+  real avg, err, spread, chisq;
+  real fmin, fmax;
+  real xminmax[];
+} Result;
+
+typedef const Result cResult;
+
+#define ResultSize (sizeof(Result) + t->ndim*2*sizeof(real))
+
+typedef struct region {
+  int depth, next;
+  count isamples, cutcomp, xmajor;
+  real fmajor, fminor, vol;
+  Bounds bounds[];
+} Region;
+
+#define RegionSize (sizeof(Region) + t->ndim*sizeof(Bounds) + t->ncomp*ResultSize)
+
+#define RegionResult(r) ((Result *)(r->bounds + t->ndim))
+
+#define RegionPtr(n) ((Region *)((char *)t->region + (n)*regionsize))
+
+
 typedef int (*Integrand)(ccount *, creal *, ccount *, real *, void *, cint *);
 
 typedef void (*PeakFinder)(ccount *, cBounds *, number *, real *);
@@ -95,42 +129,26 @@ typedef struct _this {
   Totals *totals;
   Rule rule7, rule9, rule11, rule13;
   RNGState rng;
-  void *voidregion;
+  Region *region;
   jmp_buf abort;
 } This;
 
 typedef const This cThis;
 
-#define TYPEDEFREGION \
-  typedef struct { \
-    real avg, err, spread, chisq; \
-    real fmin, fmax; \
-    real xmin[NDIM], xmax[NDIM]; \
-  } Result; \
-  typedef const Result cResult; \
-  typedef struct region { \
-    int depth, next; \
-    count isamples, cutcomp, xmajor; \
-    real fmajor, fminor, vol; \
-    Bounds bounds[NDIM]; \
-    Result result[NCOMP]; \
-  } Region
-
-#define RegionPtr(n) (&((Region *)t->voidregion)[n])
 
 #define CHUNKSIZE 4096
 
 #define AllocRegions(t) \
-  MemAlloc((t)->voidregion, (t)->size*sizeof(Region))
+  MemAlloc((t)->region, (t)->size*regionsize)
 
 #define EnlargeRegions(t, n) if( (t)->nregions + n > (t)->size ) \
-  ReAlloc((t)->voidregion, ((t)->size += CHUNKSIZE)*sizeof(Region))
+  ReAlloc((t)->region, ((t)->size += CHUNKSIZE)*regionsize)
 
 #define SAMPLERDEFS \
-  TYPEDEFREGION; \
+  csize_t regionsize = RegionSize; \
   Region *region = RegionPtr(iregion); \
   cBounds *b = region->bounds; \
-  Result *r = region->result; \
+  Result *res = RegionResult(region); \
   cSamples *samples = &t->samples[region->isamples]; \
   real *x = samples->x, *f = samples->f; \
   cnumber n = samples->n
