@@ -2,7 +2,7 @@
 	Sample.c
 		the sampling step of Suave
 		this file is part of Suave
-		last modified 20 Dec 11 th
+		last modified 30 Jul 13 th
 */
 
 
@@ -14,15 +14,15 @@ typedef struct {
 
 /*********************************************************************/
 
-static void Sample(This *t, cnumber nnew, void *voidregion,
+static void Sample(This *t, cnumber nnew, Region *region,
   real *lastw, real *lastx, real *lastf)
 {
-  TYPEDEFREGION;
-
-  Region *const region = (Region *)voidregion;
-  count comp, dim, df;
+  count comp, df;
   number n;
-  Cumulants cumul[NCOMP];
+  Vector(Cumulants, cumul, NCOMP);
+  Cumulants *C = cumul + t->ncomp, *c;
+  Bounds *bounds = RegionBounds(region), *B = bounds + t->ndim, *b;
+  Result *res;
   char **ss = NULL, *s = NULL;
   ccount chars = 128*(region->div + 1);
 
@@ -35,8 +35,7 @@ static void Sample(This *t, cnumber nnew, void *voidregion,
 
     t->rng.getrandom(t, f);
 
-    for( dim = 0; dim < t->ndim; ++dim ) {
-      cBounds *b = &region->bounds[dim];
+    for( b = bounds; b < B; ++b ) {
       creal pos = *f*NBINS;
       ccount ipos = (count)pos;
       creal prev = (ipos == 0) ? 0 : b->grid[ipos - 1];
@@ -53,7 +52,7 @@ static void Sample(This *t, cnumber nnew, void *voidregion,
 
   w[-1] = -w[-1];
   lastw = w;
-  w = region->w;
+  w = RegionW(region);
   region->n = lastw - w;
 
   if( VERBOSE > 2 ) {
@@ -67,7 +66,7 @@ static void Sample(This *t, cnumber nnew, void *voidregion,
     }
   }
 
-  Zap(cumul);
+  FClear(cumul);
   df = n = 0;
 
   while( w < lastw ) {
@@ -75,9 +74,7 @@ static void Sample(This *t, cnumber nnew, void *voidregion,
     creal weight = fabs(*w++);
     ++n;
 
-    for( comp = 0; comp < t->ncomp; ++comp ) {
-      Cumulants *c = &cumul[comp];
-
+    for( c = cumul, comp = 0; c < C; ++c ) {
       creal wfun = weight*(*f++);
       c->sum += wfun;
       c->sqsum += Sq(wfun);
@@ -115,23 +112,21 @@ static void Sample(This *t, cnumber nnew, void *voidregion,
 
   region->df = --df;
 
-  for( comp = 0; comp < t->ncomp; ++comp ) {
-    Result *r = &region->result[comp];
-    Cumulants *c = &cumul[comp];
+  for( c = cumul, res = region->result; c < C; ++c, ++res ) {
     creal sigsq = 1/c->weightsum;
     creal avg = sigsq*c->avgsum;
 
     if( LAST ) {
-      r->sigsq = 1/c->weight;
-      r->avg = r->sigsq*c->avg;
+      res->sigsq = 1/c->weight;
+      res->avg = res->sigsq*c->avg;
     }
     else {
-      r->sigsq = sigsq;
-      r->avg = avg;
+      res->sigsq = sigsq;
+      res->avg = avg;
     }
-    r->err = sqrt(r->sigsq);
+    res->err = sqrt(res->sigsq);
 
-    r->chisq = (sigsq < .9*NOTZERO) ? 0 : c->chisqsum - avg*c->chisum;
+    res->chisq = (sigsq < .9*NOTZERO) ? 0 : c->chisqsum - avg*c->chisum;
       /* This catches the special case where the integrand is constant
          over the entire region.  Unless that constant is zero, only the
          first set of samples will have zero variance, and hence weight
@@ -148,19 +143,17 @@ static void Sample(This *t, cnumber nnew, void *voidregion,
   if( VERBOSE > 2 ) {
     char *p = s;
     char *p0 = p + t->ndim*64;
+    char *msg = "\nRegion (" REALF ") - (" REALF ")";
 
-    for( dim = 0; dim < t->ndim; ++dim ) {
-      cBounds *b = &region->bounds[dim];
-      p += sprintf(p, 
-        (dim == 0) ? "\nRegion (" REALF ") - (" REALF ")" :
-                     "\n       (" REALF ") - (" REALF ")",
-        b->lower, b->upper);
+    for( b = bounds; b < B; ++b ) {
+      p += sprintf(p, msg, b->lower, b->upper);
+      msg = "\n       (" REALF ") - (" REALF ")";
     }
 
-    for( comp = 0; comp < t->ncomp; ++comp ) {
-      cResult *r = &region->result[comp];
+    for( comp = 0, res = region->result;
+         comp < t->ncomp; ++comp, ++res ) {
       p += sprintf(p, "%s  \tchisq " REAL " (" COUNT " df)",
-        p0, r->chisq, df);
+        p0, res->chisq, df);
       p0 += chars;
     }
 
